﻿#convert_csv_yml.py
#Convert a .csv file to a GSS yml .file
#Copyright (c) 2021 VideoRay LLC
#All rights reserved
version = "0.00.01 BETA"

import sys
import fileinput
import math
import uuid
from datetime import datetime
import time

python_version = int(sys.version[0])

if python_version == 3:
  import tkinter as tk
  from tkinter import *
  from tkinter.filedialog import askopenfilename
  from tkinter.filedialog import asksaveasfilename
  from tkinter import ttk
  from tkinter import colorchooser

else:
  import Tkinter as tk
  from Tkinter import *
  import tkFileDialog
  from tkFileDialog import askopenfilename
  from tkFileDialog import asksaveasfilename
  import ttk
  from Tkinter import colorchooser

#=================
def inFile_open():
#=================

  global inFile
  global ent_inFile

  inFile = askopenfilename(title="Select the .cvl file...",initialfile=inFile) # show an "Open" dialog box and return the path to the selected file
  if inFile != "":
    ent_inFile.delete(0,END)
    ent_inFile.insert(0,inFile)
  #print ("Input File:", inFile)

#=================
def update_inFile_edit():
#=================

  global inFile
  global ent_inFile

  inFile = ent_inFile.get()
  ent_inFile.bind("<KeyRelease>",event_inFile_edit)
  #print ("Input File:", inFile)

#=================
def event_inFile_edit(event):
#=================

  update_inFile_edit()

#=================
def outFile_open():
#=================

  global outFile
  global ent_outFile

  outFile = asksaveasfilename(title="Select the GSS .yml file...",initialfile=outFile) # show an "Open" dialog box and return the path to the selected file
  if outFile != "":
    ent_outFile.delete(0,END)
    ent_outFile.insert(0,inFile)
  #print ("Output File:", outFile)

#=================
def update_outFile_edit():
#=================

  global outFile
  global ent_outFile

  outFile = ent_outFile.get()
  ent_outFile.bind("<KeyRelease>",event_outFile_edit)
  #nt ("Output File:", outFile)

#=================
def event_outFile_edit(event):
#=================

  update_outFile_edit()

#=================
def validateTF(testField,testValue,validateSource,validateError):
#=================

  if testValue[0].lower() != "t" and testValue[0].lower() != "f":
    fieldError = testField + " must be True or False.  Current value = " + testValue + validateSource + testField + "."
    validateError = validateError + fieldError + "\n\n"
  return validateError

#=================
def validateUnitsType(testValue,validateSource,validateError):
#=================

  if testValue[0].lower() != "f" and testValue[0].lower() != "m":
    fieldError =  "Units Type must be Feet or Meters.  Current value = " + testValue + validateSource + " Units Type."
    validateError = validateError + fieldError + "\n\n"
  return validateError

#=================
def validateDegreesFormat(testValue,validateSource,validateError):
#=================

  if testValue.lower() != "dd" and testValue.lower() != "ddm" and testValue.lower() != "dms":
    fieldError = "Degrees Format must be DD, DDM or DMS.  Current value = " + testValue + validateSource + " Degrees Format."
    validateError = validateError + fieldError + "\n\n"
  return validateError

#=================
def validateHeadingMode(testValue,validateSource,validateError):
#=================

  if testValue.lower() != "none" and testValue.lower() != "along_line" and testValue.lower() != "fixed":
    fieldError = "Heading Mode must be None, ALONG_LINE or FIXED.  Current value = " + testValue + validateSource + "Heading Mode."
    validateError = validateError + fieldError + "\n\n"
  return validateError

#=================
def validateColor(testField,testValue,validateSource,validateError):
#=================

  result = "true"
  if testValue[0] != "#" or len(testValue) != 7:
    result = "false"
  else:
    try:
      int(testValue[1:],16)
      result = "true"
    except:
      result = "false"
  if result == "false":
    fieldError = testField + " must be in Hexidecimal format, like #FFFFFF.  Current value = " + testValue + "\n\nColor has been temporarily set to #00FFFF (blue). " + validateSource + testField
    validateError = validateError + fieldError + "\n\n"
  return validateError

#=================
def validateNumber(testField,testValue,low,high,validateSource,validateError):
#=================

  try:
    testValue = float(testValue)
    result="pass"
  except:
    result = "fail"
  if result == "pass":
    if testValue < float(low) or testValue > float(high):
      fieldError = testField + " must be a number between " + str(low) + " and " + str(high) + ".  Current value = " + str(testValue) + validateSource + testField + "."
      validateError = validateError + fieldError + "\n\n"
  else:
    fieldError = testField + " must be a number between " + str(low) + " and " + str(high) + ".  Current value = " + str(testValue) + validateSource + testField + "."
    validateError = validateError + fieldError + "\n\n"
  return validateError

#=================
def read_cfg_file():
#=================

  global inFile
  global outFile
  global unitsType
  global degreesFormat
  global autoIncrement
  global missionNameDefault
  global missionColorDefault
  global missionColorText
  global missionColorValue
  global missionColorPropagateDefault
  global missionLockedDefault
  global defaultCount
  global defaultChoice
  global waypointDefault
  global waypointColumn
  global waypointColumnCount
  global waypointColorText
  global waypointColorValue
  global showHelp

  defaultCount = 0
  defaultChoice = []
  waypointDefault = {}
  waypointColumn = []
  valuex = {}

  validateSource = "\n\nUse the Show / Set / Save Defaults button to correct "
  validateError = ""
  f_defaults = open(cfgFile,"r")
  defaultChoice = f_defaults.readlines()
  defaultCount = len(defaultChoice)
  waypointColumnCount = defaultCount - 11

#  for line in fileinput.input(cfgFile):
#    defaultChoice.append(line.strip())
#    defaultCount += 1

  showHelp = defaultChoice[0].strip().split("=")[1]
  validateError = validateTF("Show Help on Startup",showHelp,validateSource,validateError)
  inFile = defaultChoice[1].strip().split("=")[1]
  outFile = defaultChoice[2].strip().split("=")[1]

  unitsType = defaultChoice[3].strip().lower().split("=")[1]
  validateError = validateUnitsType(unitsType,validateSource,validateError)
  degreesFormat = defaultChoice[4].strip().lower().split("=")[1]
  validateError = validateDegreesFormat(degreesFormat,validateSource,validateError)
  autoIncrement = defaultChoice[5].strip().lower().split("=")[1]
  validateError = validateTF("Auto Increment Waypoint Name",autoIncrement,validateSource,validateError)

  #set mission defaults
  missionNameDefault = defaultChoice[6].strip().lower().split("=")[1]
  missionColorDefault = defaultChoice[7].strip().lower().split("=")[1]
  validateError = validateColor("Mission Color",missionColorDefault,validateSource,validateError)
  if validateError.find("Color") != "-1":
    missionColorDefault = "#00FFFF"
  missionColorText = " " + missionColorDefault + " "
  missionColorValue = missionColorDefault
  missionColorPropagateDefault = defaultChoice[8].strip().lower().split("=")[1]
  validateError = validateTF("Mission Color Propagate",missionColorPropagateDefault,validateSource,validateError)
  missionLockedDefault = defaultChoice[9].strip().lower().split("=")[1]
  validateError = validateTF("Mission Locked",missionLockedDefault,validateSource,validateError)

  for waypointData in range(11,defaultCount):
    waypointDefault[defaultChoice[waypointData].strip().lower().split("=")[0]] = defaultChoice[waypointData].strip().split("=")[1]
    waypointColumn.append(defaultChoice[waypointData].strip().lower().split("=")[0])
    if defaultCount == 26:
      waypointDefault["unused"] = "unused"

  validateError = validateColor("Waypoint Color",waypointDefault["color"],validateSource,validateError)
  validateError = validateNumber("Waypoint Tolerance",waypointDefault["tolerance"],0,6,validateSource,validateError)
  validateError = validateNumber("Waypoint Speed",waypointDefault["speed"],0,9,validateSource,validateError)
  validateError = validateTF("Depth Reference",waypointDefault["z_alt"],validateSource,validateError)
  validateError = validateTF("Use Depth",waypointDefault["z_matters"],validateSource,validateError)
  validateError = validateTF("Use Speed",waypointDefault["use_speed"],validateSource,validateError)
  validateError = validateNumber("Waypoint Effort",waypointDefault["effort"],1,100,validateSource,validateError)
  validateError = validateTF("display_rb",waypointDefault["display_rb"],validateSource,validateError)
  validateError = validateTF("Waypoint Locked",waypointDefault["locked"],validateSource,validateError)
  validateError = validateNumber("Waypoint Heading",waypointDefault["heading"],0,360,validateSource,validateError)
  validateError = validateHeadingMode(waypointDefault["heading_mode"],validateSource,validateError)

  waypointColorText = " " + waypointDefault["color"] + " "
  waypointColorValue = waypointDefault["color"]
  
  if validateError != "":
    showError (validateError)

#=================
def set_defaults():
#=================

  global inFile
  global outFile
  global unitsType
  global degreesFormat
  global autoIncrement
  global missionNameDefault
  global missionColorDefault
  global missionColorText
  global missionColorValue
  global missionColorPropagateDefault
  global missionLockedDefault
  global defaultCount
  global defaultChoice
  global waypointDefault
  global waypointColumn
  global waypointColorText
  global waypointColorValue
  global showHelp

  #=================
  def inFileDefault_select():
  #=================

    global inFile

    inFile = askopenfilename(title="Select the .cvl file...",initialfile=inFile) # show an "Open" dialog box and return the path to the selected file
    if inFile != "":
      ent_inFileDefault.delete(0,END)
      ent_inFileDefault.insert(0,inFile)
      ent_inFile.delete(0,END)
      ent_inFile.insert(0,inFile)
    #print ("Input Default File:", inFileDefault)

  #=================
  def update_inFileDefault_edit():
  #=================

    global inFile

    inFile = ent_inFileDefault.get()
    ent_inFileDefault.bind("<KeyRelease>",event_inFileDefault_edit)
    ent_inFile.delete(0,END)
    ent_inFile.insert(0,inFile)
    #print ("Input Default File:", inFile)

  #=================
  def event_inFileDefault_edit(event):
  #=================

    update_inFileDefault_edit()

   #=================
  def outFileDefault_select():
  #=================

    global outFile

    outFile = asksaveasfilename(title="Select the GSS .yml file...",initialfile=outFile) # show an "Open" dialog box and return the path to the selected file
    if outFile != "":
      ent_outFileDefault.delete(0,END)
      ent_outFileDefault.insert(0,inFile)
    #print ("Output Default File:", outFile)

  #=================
  def update_outFileDefault_edit():
  #=================

    global outFile

    outFile = ent_outFileDefault.get()
    ent_outFileDefault.bind("<KeyRelease>",event_outFileDefault_edit)
    ent_outFile.delete(0,END)
    ent_outFile.insert(0,outFile)
    #print ("Output File:", outFile)

  #=================
  def event_outFileDefault_edit(event):
  #=================

    update_outFileDefault_edit()

  #=================
  def update_unitsType():
  #=================

    global unitsType

    if rbt_unitsType.get() == 1:
      unitsType = "feet"
    elif rbt_unitsType.get() == 2:
      unitsType = "meters"
    else:
      print ("Error - Units Type")

  #=================
  def update_degreesFormat():
  #=================

    global degreesFormat

    if rbt_degreesFormat.get() == 1:
      degreesFormat = "dd"
    elif rbt_degreesFormat.get() == 2:
      degreesFormat = "ddm"
    elif rbt_degreesFormat.get() == 3:
      degreesFormat = "dms"
    else:
      print ("Error - Degrees Type")

  #=================
  def update_autoIncrement():
  #=================

    global autoIncrement

    if rbt_autoIncrement.get() == 1:
      autoIncrement = "true"
    elif rbt_autoIncrement.get() == 2:
      autoIncrement = "false"
    else:
      print ("Error - Auto Increment")

  #=================
  def update_missionName_edit():
  #=================

    global missionNameDefault

    missionNameDefault = ent_missionName.get()
    ent_missionName.bind("<KeyRelease>",event_missionName_edit)

  #=================
  def event_missionName_edit(event):
  #=================

    update_missionName_edit()

  #=================
  def choose_missionColor():
  #=================

    global missionColorDefault
    global missionColorValue

    missionColorDefault = colorchooser.askcolor(color=missionColorValue,title ="Choose Mission Color")[1]
    try:
      missionColorText = " " + missionColorDefault + " "
    except:
      print ("Error - Mission Color")
      return
    missionColorValue = missionColorDefault
    lbl_missionColorValue.configure(text=missionColorText)
    lbl_missionColorValue.configure(bg=missionColorValue)

  #=================
  def update_missionColorPropagate():
  #=================

    global missionColorPropagateDefault

    if rbt_missionColorPropagate.get() == 1:
      missionColorPropagateDefault = "true"
    elif rbt_missionColorPropagate.get()  == 2:
      missionColorPropagateDefault = "false"
    else:
      print ("Error - Mission Color Propagate")

  #=================
  def update_missionLocked():
  #=================

    global missionLockedDefault

    if rbt_missionLocked.get() == 1:
      missionLockedDefault = "true"
    elif rbt_missionLocked.get() == 2:
      missionLockedDefault = "false"
    else:
      print ("Error - Mission Locked")

  #=================
  def update_waypointName_edit():
  #=================

    global waypointDefault

    waypointDefault["name"] = ent_waypointName.get()
    ent_waypointName.bind("<KeyRelease>",event_waypointName_edit)

  #=================
  def event_waypointName_edit(event):
  #=================

    update_waypointName_edit()

  #=================
  def update_waypointY_edit():
  #=================

    global waypointDefault

    waypointDefault["y"] = ent_waypointY.get()
    ent_waypointY.bind("<KeyRelease>",event_waypointY_edit)
    #error if not degrees format

  #=================
  def event_waypointY_edit(event):
  #=================

    update_waypointY_edit()

  #=================
  def update_waypointX_edit():
  #=================

    global waypointDefault

    waypointDefault["x"] = ent_waypointX.get()
    ent_waypointX.bind("<KeyRelease>",event_waypointX_edit)
    #error if not degrees format

  #=================
  def event_waypointX_edit(event):
  #=================

    update_waypointX_edit()

  #=================
  def update_waypointZ_edit():
  #=================

    global waypointDefault

    waypointDefault["z"] = ent_waypointZ.get()
    ent_waypointZ.bind("<KeyRelease>",event_waypointZ_edit)
    #error if not + real number

  #=================
  def event_waypointZ_edit(event):
  #=================

      update_waypointZ_edit()

  #=================
  def choose_waypoint_color():
  #=================

    global waypointDefault
    global waypointColorValue

    waypointDefault["color"] = colorchooser.askcolor(color=waypointColorValue,title ="Choose Waypoint Color")[1]
    try:
      waypointColorText = " " + waypointDefault["color"] + " "
    except:
      print ("Error - Waypoint Color")
      return
    waypointColorValue = waypointDefault["color"]
    lbl_waypointColorValue.configure(text=waypointColorText)
    lbl_waypointColorValue.configure(bg=waypointColorValue)

  #=================
  def update_waypoint_tolerance_edit():
  #=================

    global waypointDefault
    waypointDefault["tolerance"] = ent_waypoint_tolerance.get()
    ent_waypoint_tolerance.bind("<KeyRelease>",event_waypoint_tolerance_edit)
    #error if not + real number

  #=================
  def event_waypoint_tolerance_edit(event):
  #=================

    update_waypoint_tolerance_edit()

  #=================
  def update_waypointZ_alt():
  #=================

    global waypointDefault

    if rbt_waypointZ_alt.get() == 1:
      waypointDefault["z_alt"] = "true"
    elif rbt_waypointZ_alt.get() == 2:
      waypointDefault["z_alt"] = "false"
    else:
      print ("Error - z_alt")

  #=================
  def update_waypointZ_matters():
  #=================

    global waypointDefault

    if rbt_waypointZ_matters.get() == 1:
      waypointDefault["z_matters"] = "true"
    elif rbt_waypointZ_matters.get() == 2:
      waypointDefault["z_matters"] = "false"
    else:
      print ("Error - z_matters")

  #=================
  def update_waypointSpeed_edit():
  #=================

    global waypointDefault

    waypointDefault["speed"] = ent_waypointSpeed.get()
    ent_waypointSpeed.bind("<KeyRelease>",event_waypointSpeed_edit)
    #error if not + real number

  #=================
  def event_waypointSpeed_edit(event):
  #=================

    update_waypointSpeed_edit()

  #=================
  def update_waypointUseSpeed():
  #=================

    global waypointDefault

    if rbt_waypointUseSpeed.get() == 1:
      waypointDefault["use_speed"] = "true"
    elif rbt_waypointUseSpeed.get() == 2:
      waypointDefault["use_speed"] = "false"
    else:
      print ("Error - Use Speed")

  #=================
  def update_waypointEffort_edit():
  #=================

    global waypointDefault

    waypointDefault["effort"] = ent_waypointEffort.get()
    ent_waypointEffort.bind("<KeyRelease>",event_waypointEffort_edit)
    #error if not + real number; 0 < effort <= 100

  #=================
  def event_waypointEffort_edit(event):
  #=================

    update_waypointEffort_edit()

  #=================
  def update_waypointDisplayRb():
  #=================

    global waypointDefault

    if rbt_waypointDisplayRb.get() == 1:
      waypointDefault["display_rb"] = "true"
    elif rbt_waypointDisplayRb.get() == 2:
      waypointDefault["display_rb"] = "false"
    else:
      print ("Error - display_rb")

  #=================
  def update_waypointLocked():
  #=================

    global waypointDefault

    if rbt_waypointLocked.get() == 1:
      waypointDefault["locked"] = "true"
    elif rbt_waypointLocked.get() == 2:
      waypointDefault["locked"] = "false"
    else:
      print ("Error - Waypoint Locked")

  #=================
  def update_waypointHeading_edit():
  #=================

    global waypointDefault

    waypointDefault["heading"] = ent_waypointHeading.get()
    ent_waypointHeading.bind("<KeyRelease>",event_waypointHeading_edit)

  #=================
  def event_waypointHeading_edit(event):
  #=================

    update_waypointHeading_edit()

  #=================
  def update_waypointHeadingMode():
  #=================

    global waypointDefault

    if rbt_waypointHeadingMode.get() == 1:
      waypointDefault["heading_mode"] = "NONE"
    elif rbt_waypointHeadingMode.get() == 2:
      waypointDefault["heading_mode"] = "ALONG_LINE"
    elif rbt_waypointHeadingMode.get() == 3:
      waypointDefault["heading_mode"] = "FIXED"
    else:
      print ("Error - Heading Mode")

  #=================
  def update_showHelp():
  #=================

    global showHelp

    if rbt_showHelp.get() == 1:
      showHelp = "true"
    elif rbt_showHelp.get() == 2:
      showHelp = "false"
    else:
      print ("Error - Show Help")

  #=================
  def reorder():
  #=================

    global waypointColumn

    #=================
    def selectRow(e):
    #=================

      selected = waypointList.focus()

    #=================
    def insert_unused():
    #=================

      global waypointColumn
      global waypointColumnCount
      global waypointDefault

      rowValues = ("Unused","unused")
      waypointList.insert(parent="", index="end", iid=waypointColumnCount, value=rowValues)
      waypointColumnCount += 1
      waypointColumn.append("unused")

    #=================
    def move_up():
    #=================

      rows = waypointList.selection()
      for row in rows:
        waypointList.move(row,waypointList.parent(row),waypointList.index(row) - 1)

    #=================
    def move_down():
    #=================

      rows = waypointList.selection()
      for row in rows:
        waypointList.move(row,waypointList.parent(row),waypointList.index(row) + 1)

    #=================
    def remove_unused():
    #=================

      global unusedValue
      global waypointColumn
      global waypointColumnCount

      rows = waypointList.focus()
      unusedValue = waypointList.item(rows,"values")
      if unusedValue[0].find("Unused") != -1:
        #update column order
        waypointColumnCount = 0
        for line in waypointList.get_children():    
          waypointColumn[waypointColumnCount] = waypointList.item(line)["values"][1]
          waypointColumnCount += 1
        waypointColumnCount -= 1
        for shift in range (int(rows),(waypointColumnCount)):
          waypointColumn[shift] = waypointColumn[(shift + 1)]
        waypointList.delete(rows)
      else:
        showError("You can only remove unused columns!")

    #=================
    def update_order():
    #=================

      global waypointColumn
      global waypointColumnCount
      
      waypointOrderNew = ""
      waypointField = 0
      for line in waypointList.get_children():     
        waypointColumn[waypointField] = waypointList.item(line)["values"][1]
        waypointOrderNew += waypointList.item(line)["values"][0] + "\n"
        waypointField += 1
      lbl_waypointOrder["text"] = waypointOrderNew
      win_reorder.destroy()

    win_reorder = Tk()
    win_reorder.title("convert_csv_yml.py Reorder Waypoint Data")

    lbl_title = Label(master=win_reorder,text="Reorder Waypoint Data",font="arial 10 bold")

    waypointList = ttk.Treeview(win_reorder,height=waypointColumnCount)
  
    waypointList["columns"]=("treeFieldName","waypointDataElement")

    waypointList["displaycolumns"] = ("treeFieldName")

    waypointList.column("#0",width=0,stretch=NO)
    waypointList.column("treeFieldName",width=200,stretch=NO)
    waypointList.column("waypointDataElement",width=200,stretch=NO)

    waypointList.heading("#0",text="",anchor=W)
    waypointList.heading("treeFieldName",text="Waypoint Data Element",anchor=W)
    waypointList.heading("waypointDataElement",text=".yml Parameter",anchor=W)
    
    waypointList.bind("<ButtonRelease-1>",selectRow)

    lbl_instructions = Label(master=win_reorder,text="To reorder the waypoint column order, select a row and\nuse the \"Move Up\" or \"Move Down\" buttons.  When finished,\nclick on the \"Update Waypoint Column Order\" button.")

    btn_insertUnused = Button(master=win_reorder,text="Insert Unused Column",command=insert_unused)
    btn_moveUp = Button(master=win_reorder,text="Move Up ^",command=move_up)
    btn_moveDown = Button(master=win_reorder,text="Move Down v",command=move_down)
    btn_removeUnused = Button(master=win_reorder,text="Remove Unused Column",command=remove_unused)
    btn_update = Button(master=win_reorder,text="Update Waypoint Column Order",command=update_order)
    btn_close = Button(master=win_reorder,text="Close without Updating",command=win_reorder.destroy)

    for waypointField in range(0,waypointColumnCount):
      rowValues = (fieldName[waypointColumn[waypointField]],waypointColumn[waypointField])
      waypointList.insert(parent="", index="end", iid=waypointField, value=rowValues)

    lbl_title.grid(row=0,column=0,columnspan=3,padx=6,pady=6)
    lbl_instructions.grid(row=1,column=0,columnspan=3,padx=10,pady=6)
    waypointList.grid(row=2,column=0,rowspan=4,padx=20,pady=6)
    btn_insertUnused.grid(row=2,column=2,padx=6,pady=6)
    btn_moveUp.grid(row=3,column=2,padx=6,pady=6)
    btn_moveDown.grid(row=4,column=2,padx=6,pady=6)
    btn_removeUnused.grid(row=5,column=2,padx=6,pady=6)
    btn_update.grid(row=6,column=0,padx=10,pady=6)
    btn_close.grid(row=6,column=2,padx=10,pady=6)

  #=================
  def helpDefaults():
  #=================

    win_helpDefaults = Tk()
    win_helpDefaults.title("convert_csv_yml.py Help Defaults")

    lbl_title = Label(master=win_helpDefaults,text="Convert .csv to GSS .yml File Help Defaults",font="arial 10 bold")
    lbl_helpDefaults = Label(master=win_helpDefaults,text=
      "A default value can be assigned to any data element.  Use the \"Show / Set / Save Defaults\" window"
       "\nto make these assignments.  When finished, click the \"Save Defaults to File\" button."
       "\nDefaults are saved to \"convert_csv_yml.cfg\""
      "\n\n"
      "The waypoint data column order can be rearranged to meet your data's column order.  Any data"
      "\nelements that you do not use should be moved to the end of the row.  These elements can then"
      "\nbe omitted from your data.  The program will pad these columns with their default values."
      "\n\n"
      "The current waypoint data column order is displayed in the \"Show / Set / Save Defaults\" window."
      "\n\n"
      "To reorder the waypoint data columns, click on the \"Reorder Waypoint Data\" button."
      "\n\n"
      "Changes made to the defaults or column order are active for the current execution."
      "\nTo update the configuration file for future executions, use the \"Save Defaults to File\" button."
      "\n\n"
      "Contact VideoRay Customer Care for assistance:  support@videoray.com"
      ,justify=LEFT)
    btn_close_helpDefaults=Button(master=win_helpDefaults,text="Close",font="arial 10 bold",command=win_helpDefaults.destroy)

    lbl_title.pack(padx=10,pady=10)
    lbl_helpDefaults.pack(padx=10,pady=10)
    btn_close_helpDefaults.pack(pady=10)

  #=================
  def save_defaults():
  #=================

    global waypointColumn

    cfgFileData = ""
    cfgFileData += "showHelp=" + showHelp + "\n"
    cfgFileData += "inFile=" + inFile + "\n"
    cfgFileData += "outFile=" + outFile + "\n"
    cfgFileData += "unitsType=" + unitsType + "\n"
    cfgFileData += "degreesFormat=" + degreesFormat + "\n"
    cfgFileData += "autoIncrement=" + autoIncrement + "\n"
    cfgFileData += "mission_name=" + missionNameDefault + "\n"
    cfgFileData += "mission_color=" + missionColorDefault + "\n"
    cfgFileData += "mission_color_propagate=" + missionColorPropagateDefault + "\n"
    cfgFileData += "mission_locked=" + missionLockedDefault + "\n"
    cfgFileData += "--You may change any of the =values in this file to set your own defaults, and you may change the sequence below this line to match your waypoint field order and fields used\n"
    for waypointField in range(0,waypointColumnCount):
      cfgFileData += waypointColumn[waypointField] + "=" + waypointDefault[waypointColumn[waypointField]] + "\n"

    #write cfgFile
    cfgFileNum = open(cfgFile,"w")
    cfgFileNum.write(cfgFileData)
    cfgFileNum.close()

    #close window
    win_defaults.destroy()

  win_defaults = Tk()
  win_defaults.title("convert_csv_yml.py Show / Set / Save Defaults")

  fieldName={}
  fieldName["name"] =         "Name (name=)                                "
  fieldName["y"] =            "North (y=)                                        "
  fieldName["x"] =            "East (x=)                                           "
  fieldName["z"] =            "Depth or Altitude (z=)                     "
  fieldName["color"] =        "Color (color=)                                   "
  fieldName["tolerance"] =    "Tolerance (tolerance=)                    "
  fieldName["z_alt"] =        "Depth Reference (z_alt=)                "
  fieldName["z_matters"] =    "Use Depth (z_matters=)                  "
  fieldName["speed"] =        "Speed (speed=)                                "
  fieldName["use_speed"] =    "Use Speed (use_speed=)                 "
  fieldName["display_rb"] =   "Display_rb (display_rb=)                 "
  fieldName["effort"] =       "Effort (effort=)                                 "
  fieldName["locked"] =       "Locked (locked=)                            "
  fieldName["heading"] =      "Heading (heading=)                       "
  fieldName["heading_mode"] = "Heading Mode (heading_mode=)"
  fieldName["unused"] =       "Unused                 "

  waypointOrder = ""
  for waypointField in range(0,waypointColumnCount):
    waypointOrder += fieldName[waypointColumn[waypointField]] + "\n"

  lbl_title = Label(master=win_defaults,text="Convert .csv to GSS .yml Show / Set /Save Defaults",font="arial 12 bold")

  lbl_inFileDefault = Label(master=win_defaults,text="Read from:")
  ent_inFileDefault = Entry(master=win_defaults,width=20)
  btn_inFileDefault = Button(master=win_defaults,text="Select File", command=inFileDefault_select)
  lbl_outFileDefault = Label(master=win_defaults,text="Write to:")
  ent_outFileDefault = Entry(master=win_defaults,width=20)
  btn_outFileDefault = Button(master=win_defaults,text="Select File", command=outFileDefault_select)
  lbl_unitsType = Label(master=win_defaults,text="Units Type:")
  rbt_unitsType = IntVar(win_defaults)
  rbt_unitsType_1 = Radiobutton(master=win_defaults,text="Feet",variable=rbt_unitsType,value=1,command=update_unitsType)
  rbt_unitsType_2 = Radiobutton(master=win_defaults,text="Meters",variable=rbt_unitsType,value=2,command=update_unitsType)
  lbl_degreesFormat = Label(master=win_defaults,text="Degrees Format:")
  rbt_degreesFormat = IntVar(win_defaults)
  rbt_degreesFormat_1 = Radiobutton(master=win_defaults,text="D.DDD",variable=rbt_degreesFormat,value=1,command=update_degreesFormat)
  rbt_degreesFormat_2 = Radiobutton(master=win_defaults,text="D M.MMM",variable=rbt_degreesFormat,value=2,command=update_degreesFormat)
  rbt_degreesFormat_3 = Radiobutton(master=win_defaults,text="D M S.SSS",variable=rbt_degreesFormat,value=3,command=update_degreesFormat)

  sep_mission = ttk.Separator(master=win_defaults,orient="horizontal")

  lbl_missionName = Label(master=win_defaults,text="Mission Name:")
  ent_missionName = Entry(master=win_defaults,width=20)
  lbl_missionColor = Label(master=win_defaults,text="Mission Color:")
  lbl_missionColorValue= Label(master=win_defaults,text=missionColorText,bg=missionColorValue)
  btn_missionColor = Button(master=win_defaults,text="Select Mission Color",command=choose_missionColor)
  lbl_missionColorPropagate = Label(master=win_defaults,text="Mission Color Propagate:")
  rbt_missionColorPropagate = IntVar(win_defaults)
  rbt_missionColorPropagate_1 = Radiobutton(master=win_defaults,text="True",variable=rbt_missionColorPropagate,value=1,command=update_missionColorPropagate)
  rbt_missionColorPropagate_2 = Radiobutton(master=win_defaults,text="False",variable=rbt_missionColorPropagate,value=2,command=update_missionColorPropagate)
  lbl_missionLocked = Label(master=win_defaults,text="Mission Locked:")
  rbt_missionLocked = IntVar(win_defaults)
  rbt_missionLocked_1 = Radiobutton(master=win_defaults,text="True",variable=rbt_missionLocked,value=1,command=update_missionLocked)
  rbt_missionLocked_2 = Radiobutton(master=win_defaults,text="False",variable=rbt_missionLocked,value=2,command=update_missionLocked)

  sep_waypoint = ttk.Separator(master=win_defaults,orient="horizontal")

  lbl_waypointName = Label(master=win_defaults,text="Waypoint Prefix:")
  ent_waypointName = Entry(master=win_defaults,width=20)
  lbl_autoIncrement = Label(master=win_defaults,text="Auto Increment Waypoint Name:")
  rbt_autoIncrement = IntVar(win_defaults)
  rbt_autoIncrement_1 = Radiobutton(master=win_defaults,text="True",variable=rbt_autoIncrement,value=1,command=update_autoIncrement)
  rbt_autoIncrement_2 = Radiobutton(master=win_defaults,text="False",variable=rbt_autoIncrement,value=2,command=update_autoIncrement)
  lbl_waypointYXZ = Label(master=win_defaults,text="North, East, Depth/Alt (y,x,z):")
  ent_waypointY = Entry(master=win_defaults,width=20)
  ent_waypointX = Entry(master=win_defaults,width=20)
  ent_waypointZ = Entry(master=win_defaults,width=10)
  lbl_waypointColor = Label(master=win_defaults,text="Waypoint Color:")
  lbl_waypointColorValue = Label(master=win_defaults,text=waypointColorText,bg=waypointColorValue)
  btn_waypoint_color = Button(master=win_defaults,text="Select Waypoint Color",command=choose_waypoint_color)
  lbl_waypointTolerance = Label(master=win_defaults,text="Waypoint Tolerance:")
  ent_waypoint_tolerance = Entry(master=win_defaults,width=10)
  lbl_waypointZ_alt = Label(master=win_defaults,text="Depth Reference:",)
  rbt_waypointZ_alt = IntVar(win_defaults)
  rbt_waypointZ_alt_1 = Radiobutton(master=win_defaults,text="Depth",variable=rbt_waypointZ_alt,value=1,command=update_waypointZ_alt)
  rbt_waypointZ_alt_2 = Radiobutton(master=win_defaults,text="Altitude",variable=rbt_waypointZ_alt,value=2,command=update_waypointZ_alt)
  lbl_waypointZ_matters = Label(master=win_defaults,text="Use Depth:")
  rbt_waypointZ_matters = IntVar(win_defaults)
  rbt_waypointZ_matters_1 = Radiobutton(master=win_defaults,text="True",variable=rbt_waypointZ_matters,value=1,command=update_waypointZ_matters)
  rbt_waypointZ_matters_2 = Radiobutton(master=win_defaults,text="False",variable=rbt_waypointZ_matters,value=2,command=update_waypointZ_matters)
  lbl_waypointSpeed = Label(master=win_defaults,text="Speed:")
  ent_waypointSpeed = Entry(master=win_defaults,width=10)
  lbl_waypointUseSpeed = Label(master=win_defaults,text="Use Speed:")
  rbt_waypointUseSpeed = IntVar(win_defaults)
  rbt_waypointUseSpeed_1 = Radiobutton(master=win_defaults,text="True",variable=rbt_waypointUseSpeed,value=1,command=update_waypointUseSpeed)
  rbt_waypointUseSpeed_2 = Radiobutton(master=win_defaults,text="False",variable=rbt_waypointUseSpeed,value=2,command=update_waypointUseSpeed)
  lbl_waypointEffort = Label(master=win_defaults,text="Effort:",height=2)
  ent_waypointEffort = Entry(master=win_defaults,width=10)
  lbl_waypointDisplayRb = Label(master=win_defaults,text="Display RB:")
  rbt_waypointDisplayRb = IntVar(win_defaults)
  rbt_waypointDisplayRb_1 = Radiobutton(master=win_defaults,text="True",variable=rbt_waypointDisplayRb,value=1,command=update_waypointDisplayRb)
  rbt_waypointDisplayRb_2 = Radiobutton(master=win_defaults,text="False",variable=rbt_waypointDisplayRb,value=2,command=update_waypointDisplayRb)
  lbl_waypointLocked = Label(master=win_defaults,text="Waypoint Locked:")
  rbt_waypointLocked = IntVar(win_defaults)
  rbt_waypointLocked_1 = Radiobutton(master=win_defaults,text="True",variable=rbt_waypointLocked,value=1,command=update_waypointLocked)
  rbt_waypointLocked_2 = Radiobutton(master=win_defaults,text="False",variable=rbt_waypointLocked,value=2,command=update_waypointLocked)
  lbl_waypointHeading = Label(master=win_defaults,text="Heading:",height=2)
  ent_waypointHeading = Entry(master=win_defaults,width=10)
  lbl_waypointHeadingMode = Label(master=win_defaults,text="Heading Mode:")
  rbt_waypointHeadingMode = IntVar(win_defaults)
  rbt_waypointHeadingMode_1 = Radiobutton(master=win_defaults,text="None",variable=rbt_waypointHeadingMode,value=1,command=update_waypointHeadingMode)
  rbt_waypointHeadingMode_2 = Radiobutton(master=win_defaults,text="Along Line",variable=rbt_waypointHeadingMode,value=2,command=update_waypointHeadingMode)
  rbt_waypointHeadingMode_3 = Radiobutton(master=win_defaults,text="Fixed",variable=rbt_waypointHeadingMode,value=3,command=update_waypointHeadingMode)

  lbl_showHelp = Label(master=win_defaults,text="Show Help On Startup:")
  rbt_showHelp = IntVar(win_defaults)
  rbt_showHelp_1 = Radiobutton(master=win_defaults,text="True",variable=rbt_showHelp,value=1,command=update_showHelp)
  rbt_showHelp_2 = Radiobutton(master=win_defaults,text="False",variable=rbt_showHelp,value=2,command=update_showHelp)

  lbl_waypointOrderHeading = Label(master=win_defaults,text="Waypoint Data Order",font="arial 10 bold")
  lbl_waypointOrder = Label(master=win_defaults,text=waypointOrder)
  lbl_waypointReorder = Label(master=win_defaults,text="This order can be changed.")
  btn_waypointReorder = Button(master=win_defaults,text="Reorder Waypoint Data",command=reorder)

  sep_action = ttk.Separator(master=win_defaults,orient="horizontal")

  btn_helpDefaults = Button(master=win_defaults,text="Help with Defaults",command=helpDefaults)
  btn_update_defaults = Button(master=win_defaults,text="Update Active Defaults",command=win_defaults.destroy)
  btn_save = Button(master=win_defaults,text="Save Defaults to File",command=save_defaults)

  #update widget values with configuration file or current information
  ent_inFileDefault.insert(0,inFile)
  ent_inFileDefault.bind("<KeyRelease>",event_inFileDefault_edit)

  ent_outFileDefault.insert(0,outFile)
  ent_outFileDefault.bind("<KeyRelease>",event_outFileDefault_edit)

  if unitsType[0].lower() == "f":
    rbt_unitsType.set(1)
  elif unitsType[0].lower() == "m":
    rbt_unitsType.set(2)

  if degreesFormat.lower() == "dd":
    rbt_degreesFormat.set(1)
  elif degreesFormat.lower() == "ddm":
    rbt_degreesFormat.set(2)
  elif degreesFormat.lower() == "dms":
    rbt_degreesFormat.set(3)

  if autoIncrement[0].lower() == "t":
    rbt_autoIncrement.set(1)
  elif autoIncrement[0].lower() == "f":
    rbt_autoIncrement.set(2)

  ent_missionName.insert(0,missionNameDefault)
  ent_missionName.bind("<KeyRelease>",event_missionName_edit)

  missionColorPropagate=missionColorPropagateDefault
  if missionColorPropagate[0].lower() == "t":
    rbt_missionColorPropagate.set(1)
  elif missionColorPropagate[0].lower() == "f":
    rbt_missionColorPropagate.set(2)

  missionLocked=missionLockedDefault
  if missionLocked[0].lower() == "t":
    rbt_missionLocked.set(1)
  elif missionLocked[0].lower() == "f":
    rbt_missionLocked.set(2)

  ent_waypointName.insert(0,waypointDefault["name"])
  ent_waypointName.bind("<KeyRelease>",event_waypointName_edit)

  ent_waypointY.insert(0,waypointDefault["y"])
  ent_waypointY.bind("<KeyRelease>",event_waypointY_edit)

  ent_waypointX.insert(0,waypointDefault["x"])
  ent_waypointX.bind("<KeyRelease>",event_waypointX_edit)

  ent_waypointZ.insert(0,waypointDefault["z"])
  ent_waypointZ.bind("<KeyRelease>",event_waypointZ_edit)

  ent_waypoint_tolerance.insert(0,waypointDefault["tolerance"])
  ent_waypoint_tolerance.bind("<KeyRelease>",event_waypoint_tolerance_edit)

  waypointZ_alt=waypointDefault["z_alt"]
  if waypointZ_alt[0].lower() == "t":
    rbt_waypointZ_alt.set(2)
  elif waypointZ_alt[0].lower() == "f":
    rbt_waypointZ_alt.set(1)

  waypointZ_matters=waypointDefault["z_matters"]
  if waypointZ_matters[0].lower() == "t":
    rbt_waypointZ_matters.set(1)
  elif waypointZ_matters[0].lower() == "f":
    rbt_waypointZ_matters.set(2)

  ent_waypointSpeed.insert(0,waypointDefault["speed"])
  ent_waypointSpeed.bind("<KeyRelease>",event_waypointSpeed_edit)

  waypointUseSpeed=waypointDefault["use_speed"]
  if waypointUseSpeed[0].lower() == "t":
    rbt_waypointUseSpeed.set(1)
  elif waypointUseSpeed[0].lower() == "f":
    rbt_waypointUseSpeed.set(2)

  ent_waypointEffort.insert(0,waypointDefault["effort"])
  ent_waypointEffort.bind("<KeyRelease>",event_waypointEffort_edit)

  waypointDisplayRb=waypointDefault["display_rb"]
  if waypointDisplayRb[0].lower() == "t":
    rbt_waypointDisplayRb.set(1)
  elif waypointDisplayRb[0].lower() == "f":
    rbt_waypointDisplayRb.set(2)

  waypointLocked=waypointDefault["locked"]
  if waypointLocked[0].lower() == "t":
    rbt_waypointLockedset(1)
  elif waypointLocked[0].lower() == "f":
    rbt_waypointLocked.set(2)

  ent_waypointHeading.insert(0,waypointDefault["heading"])
  ent_waypointHeading.bind("<KeyRelease>",event_waypointHeading_edit)

  waypointHeadingMode=waypointDefault["heading_mode"]
  if waypointHeadingMode[0].lower() == "n":
    rbt_waypointHeadingMode.set(1)
  elif waypointHeadingMode[0].lower() == "a":
    rbt_waypointHeadingMode.set(2)
  elif waypointHeadingMode[0].lower() == "f":
    rbt_waypointHeadingMode.set(3)

  if showHelp[0].lower() == "t":
    rbt_showHelp.set(1)
  elif showHelp[0].lower() == "f":
    rbt_showHelp.set(2)

  #display window
  lbl_title.grid(row=0,column=0,columnspan=5,pady=4)

  lbl_inFileDefault.grid(row=1,column=0,sticky=W,padx=3)
  ent_inFileDefault.grid(row=1,column=1,sticky=W,padx=3)
  btn_inFileDefault.grid(row=1,column=2,sticky=W,padx=3)
  lbl_outFileDefault.grid(row=2,column=0,sticky=W,padx=3)
  ent_outFileDefault.grid(row=2,column=1,sticky=W,padx=3)
  btn_outFileDefault.grid(row=2,column=2,sticky=W,padx=3)
  lbl_unitsType.grid(row=3,column=0,sticky=W,padx=3)
  rbt_unitsType_1.grid(row=3,column=1,sticky=W,padx=3)
  rbt_unitsType_2.grid(row=3,column=2,sticky=W,padx=3)
  lbl_degreesFormat.grid(row=4,column=0,sticky=W,padx=3)
  rbt_degreesFormat_1.grid(row=4,column=1,sticky=W,padx=3)
  rbt_degreesFormat_2.grid(row=4,column=2,sticky=W,padx=3)
  rbt_degreesFormat_3.grid(row=4,column=3,sticky=W,padx=3)

  sep_mission.grid(row=5,column=0,columnspan=5,sticky=EW,padx=5,pady=4)

  lbl_missionName.grid(row=6,column=0,sticky=W,padx=3)
  ent_missionName.grid(row=6,column=1,sticky=W,padx=3)
  lbl_missionColor.grid(row=7,column=0,sticky=W,padx=3)
  lbl_missionColorValue.grid(row=7,column=1,sticky=W,padx=3)
  btn_missionColor.grid(row=7,column=2,sticky=W,padx=3,pady=6)
  lbl_missionColorPropagate.grid(row=8,column=0,sticky=W,padx=3)
  rbt_missionColorPropagate_1.grid(row=8,column=1,sticky=W,padx=3)
  rbt_missionColorPropagate_2.grid(row=8,column=2,sticky=W,padx=3)
  lbl_missionLocked.grid(row=9,column=0,sticky=W,padx=3)
  rbt_missionLocked_1.grid(row=9,column=1,sticky=W,padx=3)
  rbt_missionLocked_2.grid(row=9,column=2,sticky=W,padx=3)

  sep_waypoint.grid(row=10,column=0,columnspan=5,sticky=EW,padx=5,pady=4)

  lbl_waypointName.grid(row=11,column=0,sticky=W,padx=3)
  ent_waypointName.grid(row=11,column=1,sticky=W,padx=3)
  lbl_autoIncrement.grid(row=12,column=0,sticky=W,padx=3)
  rbt_autoIncrement_1.grid(row=12,column=1,sticky=W,padx=3)
  rbt_autoIncrement_2.grid(row=12,column=2,sticky=W,padx=3)
  lbl_waypointYXZ.grid(row=13,column=0,sticky=W,padx=3)
  ent_waypointY.grid(row=13,column=1,sticky=W,padx=3)
  ent_waypointX.grid(row=13,column=2,sticky=W,padx=3)
  ent_waypointZ.grid(row=13,column=3,sticky=W,padx=3)
  lbl_waypointColor.grid(row=14,column=0,sticky=W,padx=3)
  lbl_waypointColorValue.grid(row=14,column=1,sticky=W,padx=3)
  btn_waypoint_color.grid(row=14,column=2,sticky=W,padx=3,pady=2)
  lbl_waypointTolerance.grid(row=15,column=0,sticky=W,padx=3)
  ent_waypoint_tolerance.grid(row=15,column=1,sticky=W,padx=3)
  lbl_waypointZ_alt.grid(row=16,column=0,sticky=W,padx=3)
  rbt_waypointZ_alt_1.grid(row=16,column=1,sticky=W,padx=3)
  rbt_waypointZ_alt_2.grid(row=16,column=2,sticky=W,padx=3)
  lbl_waypointZ_matters.grid(row=17,column=0,sticky=W,padx=3)
  rbt_waypointZ_matters_1.grid(row=17,column=1,sticky=W,padx=3)
  rbt_waypointZ_matters_2.grid(row=17,column=2,sticky=W,padx=3)
  lbl_waypointSpeed.grid(row=18,column=0,sticky=W,padx=3)
  ent_waypointSpeed.grid(row=18,column=1,sticky=W,padx=3)
  lbl_waypointUseSpeed.grid(row=19,column=0,sticky=W,padx=3)
  rbt_waypointUseSpeed_1.grid(row=19,column=1,sticky=W,padx=3)
  rbt_waypointUseSpeed_2.grid(row=19,column=2,sticky=W,padx=3)
  lbl_waypointEffort.grid(row=20,column=0,sticky=W,padx=3)
  ent_waypointEffort.grid(row=20,column=1,sticky=W,padx=3)
  lbl_waypointDisplayRb.grid(row=21,column=0,sticky=W,padx=3)
  rbt_waypointDisplayRb_1.grid(row=21,column=1,sticky=W,padx=3)
  rbt_waypointDisplayRb_2.grid(row=21,column=2,sticky=W,padx=3)
  lbl_waypointLocked.grid(row=22,column=0,sticky=W,padx=3)
  rbt_waypointLocked_1.grid(row=22,column=1,sticky=W,padx=3)
  rbt_waypointLocked_2.grid(row=22,column=2,sticky=W,padx=3)
  lbl_waypointHeading.grid(row=23,column=0,sticky=W,padx=3)
  ent_waypointHeading.grid(row=23,column=1,sticky=W,padx=3)
  lbl_waypointHeadingMode.grid(row=24,column=0,sticky=W,padx=3)
  rbt_waypointHeadingMode_1.grid(row=24,column=1,sticky=W,padx=3)
  rbt_waypointHeadingMode_2.grid(row=24,column=2,sticky=W,padx=3)
  rbt_waypointHeadingMode_3.grid(row=24,column=3,sticky=W,padx=3)

  lbl_waypointOrderHeading.grid(row=11,column=4,sticky=W,padx=3)
  lbl_waypointOrder.grid(row=11,column=4,rowspan=12,sticky=W,padx=3)
  lbl_waypointReorder.grid(row=22,column=4,sticky=W,padx=3)
  btn_waypointReorder.grid(row=23,column=4,sticky=W,padx=3)

  sep_action.grid(row=25,column=0,columnspan=5,sticky=EW,padx=5,pady=4)

  lbl_showHelp.grid(row=26,column=0,sticky=W,padx=3)
  rbt_showHelp_1.grid(row=26,column=1,sticky=W,padx=3)
  rbt_showHelp_2.grid(row=26,column=2,sticky=W,padx=3)

  btn_helpDefaults.grid(row=27,column=1,sticky=E,padx=20,pady=10)
  btn_update_defaults.grid(row=27,column=2,sticky=W,padx=6,pady=10)
  btn_save.grid(row=27,column=3,sticky=W,padx=6,pady=10)

#=================
def convert():
#=================

  global lbl_pointsData
  global unitsType
  global degreesFormat
  global autoIncrement

  missionLine = []
  waypointLine = []
  tag={}

  validateError = ""

  creation_timeTag = "<!--##creation_time##-->"
  mission_idTag = "<!--##mission_id##-->"
  mission_nameTag = "<!--##mission_name##-->"
  mission_colorTag = "<!--##mission_color##-->"
  mission_color_propagateTag = "<!--##color_propagate##-->"
  mission_lockedTag = "<!--##mission_locked##-->"

  waypoint_idTag = "<!--##waypoint_id##-->"
  tag["name"] = "<!--##waypoint_name##-->"
  tag["color"] = "<!--##waypoint_color##-->"
  tag["x"] = "<!--##x##-->"
  tag["y"] = "<!--##y##-->"
  tag["z"] = "<!--##z##-->"
  tag["tolerance"] = "<!--##tolerance##-->"
  tag["z_alt"] = "<!--##z_alt##-->"
  tag["z_matters"] = "<!--##z_matters##-->"
  tag["speed"] = "<!--##speed##-->"
  tag["use_speed"] = "<!--##use_speed##-->"
  tag["effort"] = "<!--##effort##-->"
  tag["display_rb"] = "<!--##display_rb##-->"
  tag["locked"] = "<!--##locked##-->"
  tag["heading"] = "<!--##heading##-->"
  tag["heading_mode"] = "<!--##heading_mode##-->"
  tag["ignore"] = "<!--##ignore##-->"

  missionTemplateFile = "templates/mission.tmplt"
  waypointTemplateFile = "templates/waypoint.tmplt"

  missionTemplateData = open(missionTemplateFile)
  missionBlock = missionTemplateData.read()

  waypointTemplateData = open(waypointTemplateFile)
  waypointTemplate = waypointTemplateData.read()

  fcsv = open(inFile,"r")
  csvData = fcsv.readlines()
  csvSize = len(csvData)

  missionID = str(uuid.uuid4())
  missionBlock = missionBlock.replace(creation_timeTag,datetime.now().strftime("%Y-%m-%d  %H:%M:%S"))
  missionBlock = missionBlock.replace(mission_idTag,missionID)

  missionField = csvData[0].strip().split(",")

  while len(missionField) < 7:
    missionField.append("")
  if missionField[0] == "":
    missionField[0] = missionNameDefault
  missionBlock = missionBlock.replace(mission_nameTag,missionField[0])
  if missionField[1] == "":
    missionField[1] = missionColorDefault
    validateSource = "\n\nUse Show /Set Defaults button to correct "
  else:
    validateSource = "\n\nCheck line 1 and correct "
  validateError = validateColor("Mission Color",missionField[1],validateSource,validateError)
  missionBlock = missionBlock.replace(mission_colorTag,missionField[1])
  if missionField[2] == "":
    missionField[2] = missionColorPropagateDefault
    validateSource = "\n\nUse Show /Set Defaults button to correct "
  else:
    validateSource = "\n\nCheck line 1 and correct "
  validateError = validateTF("Mission Color Propagate",missionField[2],validateSource,validateError)
  missionBlock = missionBlock.replace(mission_color_propagateTag,missionField[2])
  if missionField[3] == "":
    missionField[3] = missionLockedDefault
    validateSource = "\n\nUse Show /Set Defaults button to correct "
  else:
    validateSource = "\n\nCheck line 1 and correct "
  validateError = validateTF("Mission Locked",missionField[2],validateSource,validateError)
  missionBlock = missionBlock.replace(mission_lockedTag,missionField[3])
  if missionField[4] != "":
    unitsType = missionField[4]
    validateSource = "\n\nUse Show /Set Defaults button to correct "
  else:
    validateSource = "\n\nCheck line 1 and correct "
  validateError = validateUnitsType(unitsType,validateSource,validateError)
  if missionField[5] != "":
    degreesFormat = missionField[5]
    validateSource = "\n\nUse Show /Set Defaults button to correct "
  else:
    validateSource = "\n\nCheck line 1 and correct "
  validateError = validateDegreesFormat(degreesFormat,validateSource,validateError)
  if missionField[6] != "":
    autoIncrement = missionField[6]
    validateSource = "\n\nUse Show /Set Defaults button to correct "
  else:
    validateSource = "\n\nCheck line 1 and correct "
  validateError = validateTF("AutoIncrement",autoIncrement,validateSource,validateError)

  waypointBlocks = ""

  #print (inFile, outFile)
  #print (missionField[0], missionField[1], missionField[2], missionField[3])
  #print (unitsType, degreesFormat, autoIncrement)

  points = 0
  errorCount = 0
  errorLimit = "false"
  for waypointCount in range(1,(csvSize)):
    if csvData[waypointCount][0] != "#":
      validateError = ""
      waypointBlock = waypointTemplate
      waypointUUID = str(uuid.uuid4())
      waypointBlock = waypointBlock.replace(waypoint_idTag,waypointUUID)
      waypointBlock = waypointBlock.replace(mission_idTag,missionID)
      waypointRow = csvData[waypointCount].strip().split(",")
      while len(waypointRow) < waypointColumnCount:
        waypointRow.append("")
      for waypointField in range(0,waypointColumnCount):
        #use default if value not defined
        if waypointRow[waypointField] == "":
          waypointRow[waypointField] = waypointDefault[waypointColumn[waypointField]]
          validateSource = "\n\nUse Show /Set Defaults button to correct "
        else:
          validateSource = "\n\nCheck line " + str(waypointCount + 1) + " and correct "

        if waypointColumn[waypointField] == "name":
          #add waypoint prefix if necessary
          if waypointDefault["name"] != "":
            waypointRow[waypointField] = waypointDefault["name"] + waypointRow[waypointField]
          #add waypoint increment if necessary
          if autoIncrement[0] == "t":
            if points < 10:
              waypointNumber = "0" + str(points)
            if points < 100:
              waypointNumber = "0" + str(points)
            waypointRow[waypointField] += waypointNumber

                    #convert degress format to DD
        elif waypointColumn[waypointField] == "y" or waypointColumn[waypointField] == "x":
          if waypointRow[waypointField].find("-") != -1:
            waypointRow[waypointField] = waypointRow[waypointField].replace("-","")
            sign = "-"
          else:
            sign = ""
          if waypointRow[waypointField].find("'") != -1 or waypointRow[waypointField].find("\"") != -1 or waypointRow[waypointField].find(",") != -1:
            waypointRow[waypointField] = waypointRow[waypointField].replace("'","").replace("\"","").replace(",","").strip()
          if waypointRow[waypointField].find("e") != -1 or waypointRow[waypointField].find("E") != -1:
            waypointRow[waypointField] = waypointRow[waypointField].replace("e","").replace("E","").strip()
          if waypointRow[waypointField].find("n") != -1 or waypointRow[waypointField].find("N") != -1:
            waypointRow[waypointField] = waypointRow[waypointField].replace("n","").replace("N","").strip()
          if waypointRow[waypointField].find("w") != -1 or waypointRow[waypointField].find("W") != -1:
            waypointRow[waypointField] = waypointRow[waypointField].replace("w","").replace("W","").strip()
            sign = "-"
          if waypointRow[waypointField].find("s") != -1 or waypointRow[waypointField].find("S") != -1:
            waypointRow[waypointField] = waypointRow[waypointField].replace("s","").replace("S","").strip()
            sign = "-"
          if degreesFormat == "ddm":
            waypointRow[waypointField] = waypointRow[waypointField].replace("'","").replace("\"","")
            waypointCoord = waypointRow[waypointField].split(" ")
            if len(waypointCoord) == 2:
              waypointRow[waypointField] = str(float(waypointCoord[0]) + float(waypointCoord[1]) / 60)
            else :
              waypointRow[waypointField] = str(float(waypointCoord[0]))
          elif degreesFormat == "dms":
            waypointCoord = waypointRow[waypointField].split(" ")
            if len(waypointField) == 3:
              waypointRow[waypointField] = str(float(waypointCoord[0]) + float(waypointCoord[1] / 60) + float(waypointCoord[2]) / 360)
            elif len(waypointField) == 2:
              waypointRow[waypointField] = str(float(waypointCoord[0]) + float(waypointCoord[1] / 60))
            else:
              waypointRow[waypointField] = str(float(waypointCoord[0]))
          waypointRow[waypointField] = sign + waypointRow[waypointField]

        #convert feet to meters if necessary
        elif waypointColumn[waypointField] == "z":
          validateError = validateNumber("Depth / Altitude",waypointRow[waypointField],0,100000,validateSource,validateError)
          if unitsType[0] == "f":
            try:
              waypointRow[waypointField] = str(float(waypointRow[waypointField]) * 0.3048)
            except:
              validateError = validateError + "Invalid waypoint depth value: " + str(waypointRow[waypointField])
        
        elif waypointColumn[waypointField] == "color":
          validateError = validateColor("Waypoint Color",waypointRow[waypointField],validateSource,validateError)

        elif waypointColumn[waypointField] == "tolerance":
          validateError = validateNumber("Waypoint Tolerance",waypointRow[waypointField],0,6,validateSource,validateError)
          if unitsType[0] == "f":
            try:
              waypointRow[waypointField] = str(float(waypointRow[waypointField]) * 0.3048)
            except:
              validateError = validateError + "Invalid waypont tolerance value: " + str(waypointRow[waypointField])

        elif waypointColumn[waypointField] == "z_alt":
          validateError = validateTF("Depth Reference",waypointRow[waypointField],validateSource,validateError)

        elif waypointColumn[waypointField] == "z_matters":
          validateError = validateTF("Use Depth",waypointRow[waypointField],validateSource,validateError)

        elif waypointColumn[waypointField] == "speed":
          validateError = validateNumber("Waypoint Speed",waypointRow[waypointField],0,100,validateSource,validateError)
          if unitsType[0] == "f":
            try:
              waypointRow[waypointField] = str(float(waypointRow[waypointField]) * 0.3048)
            except:
              validateError = validateError + "Invalid waypoint speed value: " + str(waypointRow[waypointField])

        elif waypointColumn[waypointField] == "use_speed":
          validateError = validateTF("Use Speed",waypointRow[waypointField],validateSource,validateError)

        elif waypointColumn[waypointField] == "effort":
          validateError = validateNumber("Waypoint Effort",waypointRow[waypointField],1,100,validateSource,validateError)

        elif waypointColumn[waypointField] == "display_rb":
          validateError = validateTF("display_rb",waypointRow[waypointField],validateSource,validateError)

        elif waypointColumn[waypointField] == "locked":
          validateError = validateTF("Waypoint Locked",waypointRow[waypointField],validateSource,validateError)

        elif waypointColumn[waypointField] == "heading":
          validateError = validateNumber("Heading",waypointRow[waypointField],0,360,validateSource,validateError)

        elif waypointColumn[waypointField] == "headingMode":
          validateError = validateHeadingMode(waypointRow[waypointField],validateSource,validateError)

        if waypointColumn[waypointField] != "unused":
          waypointBlock = waypointBlock.replace(tag[waypointColumn[waypointField]],waypointRow[waypointField])
        

      if validateError == "":
        missionBlock += '      - id: "{' + waypointUUID + '}"\n'
        waypointBlocks += waypointBlock
        points += 1
      else:
        errorCount += 1
        if errorCount < 500:
          validateError = validateError + "\n\nWaypoint not added to mission."
          showError(validateError)
        else:
          if errorLimit != "true":
            showError("Mulitiple errors detected.\n\nAdditional errors notifications will be supressed.")
            errorLimit = "true"

  #write yml
  missionYML = missionBlock + "waypoint_data:\n" + waypointBlocks
  ymlFileNum = open(outFile,"w")
  ymlFileNum.write(missionYML)
  ymlFileNum.close()

  #update number of points
  lbl_pointsData["text"] = "    " + str(points)

#=================
def help():
#=================

  #=================
  def data_help():
  #=================

    win_help.attributes('-topmost', False)
    win_helpData = Tk()
    win_helpData.title("convert_csv-yml.py Data Help")
    lbl_title = Label(master=win_helpData,text="CSV Data Format Help",font="arial 10 bold")
    lbl_helpData = Label(master=win_helpData,text=
      "Mission Data (Row 1):"
      "\n\n  Mission Name (Text)"
      "\n  Mission Color (Hex RGB color value, range: #000000 - #FFFFFF)"
      "\n  Mission Color Propagate (True / False)"
      "\n  Mission Locked (True / False)"
      "\n  Units Type (Feet / Meters)"
      "\n  Lat/Lon Degrees Format (DD / DDM / DMS)"
      "\n      DD  = Decimal Degrees"
      "\n      DDM = Degrees Decimal Minutes"
      "\n      DMS = Degrees  Minutes Decimal Seconds"
      "\n  Auto Increment Waypoint Name (True / False)"
      "\n  "
      "\nWaypoint Data (Rows 2 - End of File):"
      "\n\n  Waypoint name (Text)"
      "\n  Waypoint y [North] (Number - Degrees value)"
      "\n  Waypoint x [East] (Number - Degrees value)"
      "\n  Waypoint z [Depth / Altitude] (Number)"
      "\n  Waypoint color (Hex RGB color value, range: #000000 - #FFFFFF)"
      "\n  Waypoint tolerance (Number)"
      "\n  Waypoint depth reference, z_alt [Z is Altitude] (True / False)"
      "\n  Waypoint use depth, z_matters [Use Depth Lock] (True / False)"
      "\n  Waypoint speed (Number)"
      "\n  Waypoint use_speed [Use Speed Lock] (True / False)"
      "\n  Waypoint effort [%] (Number, range: 0 - 100)"
      "\n  Waypoint display_rb (True / False)"
      "\n  Waypoint locked (True / False)"
      "\n  Waypoint heading (Number, range: 0 - 360)"
      "\n  Waypoint heading_mode (NONE / ALONG_LINE / FIXED)"
      "\n\nThe waypoint data elements order can be changed. Use the \"Reorder"
      "\n\"Waypoint Data\" button from the \"Show / Set /Save Defaults\" window."
      "\n\n"
      "Contact VideoRay Customer Care for assistance:  support@videoray.com"
      ,justify=LEFT)
    btn_close_data_help=Button(master=win_helpData,text="Close",font="arial 10 bold",command=win_helpData.destroy)

    lbl_title.pack(padx=10,pady=10)
    lbl_helpData.pack(padx=10,pady=10)
    btn_close_data_help.pack(pady=10)

  win_help = Tk()
  if helpShown != "true":
    win_help.attributes('-topmost', True)
  win_help.title("convert_csv_yml.py Help")
  lbl_title = Label(master=win_help,text="Convert .csv File to GSS .yml File Help",font="arial 10 bold")
  lbl_version = Label(master=win_help,text="Version " + version)
  lbl_help = Label(master=win_help,text=
    "convert_csv_yml.py converts a coordinate file in .csv format into a Greensea .yml mission file."
    "\n\n"
    "It is designed to be as easy to use as possible:"
    "\n\n   *  It can be configured to match your requirements, and does not force you to change your data"
    "\n   *  Values for any data element can be assigned as defaults"
    "\n   *  Lat/Lon Degrees format is flexible (DD, DDM, DMS; use +/- or N/S; use , ' and \" or not)"
    "\n   *  Column order can be user defined"
    "\n   *  Data columns can be omitted if the default value is acceptable"
    "\n   *  Extra data columns can be ignored"
    "\n\n"
    "Before converting your .csv file, you should click the \"Show / Set / Save Defaults\" button to set up "
    "\nthe defaults to match your data requirements and format.  After the defaults have been set and saved,"
    "\nclick on the \"Convert\" button to convert your .csv file into a GSS .yml waypoint file."
    "\n\n"
    "For more information about defaults, see the Help from the \"Show / Set /Save Defaults\" window."
    "\n\n"
    "Input .csv file format:"
    "\n\nRow 1 (Mission information):"
    "\nname, color, color_propagate, locked, units, degrees format, auto increment"
    "\n\n"
    "Rows 2 -> End of File {Waypoint information, one waypoint per row):"
    "\nWaypoints data elements column order is defined in the configuration file."
    "\n\nFor more information about the data format, click the \"CSV Data Format Help\" button."
    "\n\nYou can choose not to display this Help window on startup as one of the default settings."
    "\n\nContact VideoRay Customer Care for assistance:  support@videoray.com"
    ,justify=LEFT)
  btn_data_help=Button(master=win_help,text="CSV Data Format Help",font="arial 10 bold",command=data_help)
  btn_close_help=Button(master=win_help,text="Close",font="arial 10 bold",command=win_help.destroy)

  lbl_title.pack(padx=10,pady=10)
  lbl_version.pack()
  lbl_help.pack(padx=10,pady=10)
  btn_data_help.pack(side=LEFT,padx=60,pady=10)
  btn_close_help.pack(side=RIGHT,padx=60,pady=10)

#=================
def showError(errorMessage):
#=================

  win_errorWindow = Tk()
  win_errorWindow.title("convert_csv_yml.py - Error Message")

  lbl_title = Label(master=win_errorWindow,text="Data Error Detected",font="arial 12 bold")
  lbl_errorMessage =Label(master=win_errorWindow,text=errorMessage)
  btn_close = Button(master=win_errorWindow,text="Close",command=win_errorWindow.destroy)

  lbl_title.grid(row=0,column=0,padx=6,pady=6)
  lbl_errorMessage.grid(row=1,column=0,padx=6,pady=6)
  btn_close.grid(row=2,column=0,padx=10,pady=10)
  
#=================
def exit():
#=================

  sys.exit()

main = Tk()
main.title("convert_csv_yml.py")

cfgFile = "convert_csv_yml.cfg"
read_cfg_file()

lbl_title = Label(master=main,text="Convert .csv to GSS .yml",font="arial 12 bold")

lbl_inFile = Label(master=main,text="Read from:")
ent_inFile = Entry(master=main,width=20)
btn_inFile = Button(master=main,text="Select .csv File", command=inFile_open)
lbl_outFile = Label(master=main,text="Write to:")
ent_outFile = Entry(master=main,width=20)
btn_outFile = Button(master=main,text="Select .yml File", command=outFile_open)

lbl_points = Label(master=main,text="Points Converted:")
lbl_pointsData = Label(master=main,text= "___")

btn_help= Button(master=main,text="Help",command=help)
btn_set_defaults = Button(master=main,text="Show / Set / Save Defaults",command=set_defaults)
btn_convert = Button(master=main,text="Convert",command=convert)
btn_exit = Button(master=main,text="Exit",command=exit)

lbl_copyright = Label(master=main,text="Copyright \N{COPYRIGHT SIGN} 2021, VideoRay LLC",font="arial 7")

ent_inFile.insert(0,inFile)
ent_inFile.bind("<KeyRelease>",event_inFile_edit)

ent_outFile.insert(0,outFile)
ent_outFile.bind("<KeyRelease>",event_outFile_edit)

lbl_title.grid(row=0,column=0,columnspan=4,pady=6)

lbl_inFile.grid(row=1,column=0,sticky=E,padx=3,pady=3)
ent_inFile.grid(row=1,column=1,sticky=W,padx=3,pady=3)
btn_inFile.grid(row=1,column=2,sticky=W,padx=3,pady=3)
lbl_outFile.grid(row=2,column=0,sticky=E,padx=3,pady=3)
ent_outFile.grid(row=2,column=1,sticky=W,padx=3,pady=3)
btn_outFile.grid(row=2,column=2,sticky=W,padx=3,pady=3)

lbl_points.grid(row=3,column=1,sticky=W,padx=6,pady=3)
lbl_pointsData.grid(row=3,column=2,sticky=W,padx=6,pady=3)

btn_help.grid(row=4,column=0,sticky=W,padx=10,pady=6)
btn_set_defaults.grid(row=4,column=1,sticky=W,padx=4,pady=6)
btn_convert.grid(row=4,column=2,sticky=W,padx=6,pady=6)
btn_exit.grid(row=4,column=3,sticky=W,padx=10,pady=6)
lbl_copyright.grid(row=5,column=0,columnspan=4)

helpShown = "false"
if showHelp[0].lower() == "t":
  help()

helpShown = "true"

main.mainloop()